#ifndef MEM_H
#define MEM_H

#ifndef MEM_CHECK
# define MEM_CHECK 1
#endif

# include <stddef.h>

/* xalloc() and xfree() are wrappers for malloc and free that maintain
   a count of blocks allocated. This count is returned by numalloced(),
   so we can check for memory leaks. In addition, xalloc() checks
   the result of malloc and exits on failure.

   Compiling with -DMEM_CHECK=0 skips the allocation count for a
   probably marginal increase in efficiency, but xalloc() still
   checks its result.
*/

void *xalloc(size_t);
#if MEM_CHECK
  void xfree(void *);
  int numalloced(void);
#else
# include <stdlib.h>
# define xfree(p) free(p)
# define numalloced() 0
#endif

#endif
