#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "aa.h"
#include "mem.h"

static void usage(const char *prog)
{
    fprintf(stderr, "usage: %s [-pn] [-mM[file]]\n", prog); 
    exit(EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int i, j, nd, rev, num, mat, first;
    char *prog, *fname, *a;
    FILE *outf;

    /* Process the arguments. */
    prog = argc > 0 && **argv ? argv[0] : "table";
    fname = NULL;
    rev = num = mat = 0;
    if (argc > 0)
        ++argv;
    while ((a = *argv++)) {
        if (*a == '-') {
            int m = 0;
            while (*++a && !m) {
                switch (*a) {
                case 'p':
                    rev = 1;
                    break;
                case 'n':
                    num = 1;
                    break;
                case 'm':
                    if (!mat)
                        mat = 1;
                    m = 1;
                    break;
                case 'M':
                    mat = 2;
                    m = 1;
                    break;
                default:
                    usage(prog);
                }
                if (m && *++a) {
                    if (fname)
                        usage(prog);
                    fname = a;
                }
            }
        }
        else {
            if (fname)
                usage(prog);
            fname = a;
        }
    }
    if (fname) {
        if (!mat)
            mat = 1;
        if (!(outf = fopen(fname, "w"))) {
            fprintf(stderr, "unable to open %s (%s)\n",
                    fname, strerror(errno));
            return EXIT_FAILURE;
        }
    }
    else
        outf = stdout;
    
    init_periodic_table();
    /* Print the periodic table (maybe). */
    if (mat < 2) {
        printf("Periodic table, showing %s elements:\n",
               rev ? "parent" : "daughter");
        for (i = 92; i > 0; --i) {
            printf("%2d %-2s: ", i, elt_name(i));
            prtcmpd(elt_str(i));
            fputs("; ", stdout);
            first = 1;
            if (rev) {
                int i2;
                for (i2 = 92; i2 > 0; --i2) {
                    nd = num_dtrs(i2);
                    for (j = 1; j <= nd && get_dtr(i2, j) != i; ++j)
                        ;
                    if (j <= nd) {
                        if (first)
                            first = 0;
                        else
                            putchar(',');
                        if (num)
                            printf("%d", i2);
                        else
                            printf("%s", elt_name(i2));
                    }
                }
            }
            else {
                nd = num_dtrs(i);
                for (j = 1; j <= nd; ++j) {
                    if (first)
                        first = 0;
                    else
                        putchar('.');
                    if (num)
                        printf("%d", get_dtr(i, j));
                    else
                        printf("%s", elt_name(get_dtr(i, j)));
                }
            }
            putchar('\n');
        }
    }

    /* Print the transition matrix (maybe). */
    if (mat) {
        int row[U];
        
        if (!fname)
            puts("Transition matrix:");
        fputc('{', outf);
        for (i = 0; i < U; ++i) {
            for (j = 0; j < U; ++j)
                row[j] = 0;
            nd = num_dtrs(U - i);
            for (j = 1; j <= nd; ++j)
                ++row[U - get_dtr(U - i, j)];
            if (i > 0)
                fputs(",\n ", outf);
            fputc('{', outf);
            for (j = 0; j < U - 1; ++j) {
                fprintf(outf, "%d,", row[j]);
                if (j % 23 == 22)
                    fputs("\n  ", outf);
            }
            fprintf(outf, "%d}", row[U - 1]);
        }
        fputs("}\n", outf);
    }

    /* Cleanup. */
    if (fname)
        fclose(outf);
    kill_periodic_table();
    assert(numalloced() == 0);
    return 0;
}
